//! Constants

pub const MAX_NUM_TILES: usize = 500;
pub const MIN_NUM_TILES: usize = 100;
pub const MAX_NUM_LARGE_TILES: usize = 100;
pub const MIN_TILE_SIZE: usize = 50;
pub const MAX_MEDIAN_TILE_SIZE: usize = 750;
pub const MIN_MEDIAN_TILE_SIZE: usize = 100;
